/************************************************************************
 **
 **  @file   toggledetailinlayout.h
 **  @author Roman Telezhynskyi <dismine(at)gmail.com>
 **  @date   25 6, 2016
 **
 **  @brief
 **  @copyright
 **  This source code is part of the Valentina project, a pattern making
 **  program, whose allow create and modeling patterns of clothing.
 **  Copyright (C) 2016 Valentina project
 **  <https://bitbucket.org/dismine/valentina> All Rights Reserved.
 **
 **  Valentina is free software: you can redistribute it and/or modify
 **  it under the terms of the GNU General Public License as published by
 **  the Free Software Foundation, either version 3 of the License, or
 **  (at your option) any later version.
 **
 **  Valentina is distributed in the hope that it will be useful,
 **  but WITHOUT ANY WARRANTY; without even the implied warranty of
 **  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **  GNU General Public License for more details.
 **
 **  You should have received a copy of the GNU General Public License
 **  along with Valentina.  If not, see <http://www.gnu.org/licenses/>.
 **
 *************************************************************************/

#ifndef TOGGLEDETAILINLAYOUT_H
#define TOGGLEDETAILINLAYOUT_H

#include <qcompilerdetection.h>
#include <QMetaObject>
#include <QObject>
#include <QString>
#include <QtGlobal>

#include "vundocommand.h"

class TogglePieceInLayout : public VUndoCommand
{
    Q_OBJECT
public:
    TogglePieceInLayout(quint32 id, bool state, VContainer *data, VAbstractPattern *doc,
                        QUndoCommand *parent = nullptr);
    virtual ~TogglePieceInLayout() = default;
    virtual void undo() override;
    virtual void redo() override;
signals:
    void Toggled(quint32 id);
private:
    Q_DISABLE_COPY(TogglePieceInLayout)
    quint32     m_id;
    VContainer *m_data;
    bool        m_oldState;
    bool        m_newState;

    void Do(bool state);
};

enum class ForceForbidFlippingType : qint8 {ForceFlipping, ForbidFlipping};

class TogglePieceForceForbidFlipping : public VUndoCommand
{
    Q_OBJECT
public:
    TogglePieceForceForbidFlipping(quint32 id, bool state, ForceForbidFlippingType type, VContainer *data,
                                   VAbstractPattern *doc, QUndoCommand *parent = nullptr);
    virtual ~TogglePieceForceForbidFlipping() = default;
    virtual void undo() override;
    virtual void redo() override;
private:
    Q_DISABLE_COPY(TogglePieceForceForbidFlipping)
    quint32     m_id;
    VContainer *m_data;
    ForceForbidFlippingType m_type;
    bool        m_oldForceState;
    bool        m_newForceState;
    bool        m_oldForbidState;
    bool        m_newForbidState;
};

#endif // TOGGLEDETAILINLAYOUT_H
